#include <lpc214x.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

void delay_ms(uint16_t j) /* Function for delay in milliseconds */
{
    uint16_t x,i;
	for(i=0;i<j;i++)
	{
    for(x=0; x<6000; x++);    /* loop to generate 1 millisecond delay with Cclk = 60MHz */
	}
}

void LCD_CMD(char command)
{
	IO1SET |= 0x00400000; /* EN = 1 */
	IO1CLR |= 0x01800000; /* RS = 0, RW = 0 */
	delay_ms(5);
	IO0SET |= (command<<6);
	IO1CLR |= 0x00400000; /* EN = 0, RS and RW unchanged(i.e. RS = RW = 0)	 */
	IO0CLR |= 0x00003C00;
	delay_ms(5);
	IO1SET |= 0x00400000; /* EN = 1 */
	delay_ms(5);
	IO0SET |= (command<<10);
	IO1CLR |= 0x00400000; 
	IO0CLR |= 0x00003C00;
	delay_ms(5);
	IO0CLR |= (0xFF<<10);
	
}

void LCD_INIT(void)
{
	IO0DIR = 0x00003C00; // 3C or FF /* P0.12 to P0.15 LCD Data. P0.4,5,6 as RS RW and EN */
	IO1DIR = 0x01C00000;
	delay_ms(20);
	LCD_CMD(0x02);  /* Initialize lcd in 4-bit mode */
	LCD_CMD(0x28);  /* 2 lines */
	LCD_CMD(0x0C);   /* Display on cursor off */
	LCD_CMD(0x06);  /* Auto increment cursor */
	LCD_CMD(0x01);   /* Display clear */
	LCD_CMD(0x80);  /* First line first position */
}

void LCD_STRING (char* msg)
{
	uint8_t i=0;
	while(msg[i]!=0)
	{
		IO1SET |= 0x01400000; /* RS = 1, EN = 1 */
		IO1CLR |= 0x00800000; /* RW = 0 */
		delay_ms(2);
		IO0SET |= (msg[i]<<6);
		IO1CLR |= 0x00400000;
		IO0CLR |= 0x00003C00;
		delay_ms(5);
		IO1SET |= 0x00400000; /* RS = 1, EN = 1 "0x01400000" */ 
		IO0SET |= (msg[i]<<10);
		IO1CLR |= 0x00400000; 
		IO0CLR |= 0x00003C00;
		delay_ms(2);
		IO0CLR |= (0xFF<<10);
		i++;
	}
}

void LCD_CHAR (char msg)
{
		IO1SET |= 0x01400000; /* RS = 1, EN = 1 */
		IO1CLR |= 0x00800000; /* RW = 0 */
		delay_ms(2);
		IO0SET |= (msg<<6);
		IO1CLR |= 0x00400000; 
		IO0CLR |= 0x00003C00;
		delay_ms(5);
		IO1SET |= 0x00400000; /* RS = 1, EN = 1 "0x01400000"*/
		IO0SET |= (msg<<10);
		IO1CLR |= 0x00400000; 
		IO0CLR |= 0x00003C00;
		delay_ms(2);
		IO0CLR |= (0xFF<<10);
}

int main(void)
{
	uint32_t result;
	float voltage;
	char volt[18];
	LCD_INIT();
	LCD_STRING("   ADC VALUE   ");//first line
	LCD_CMD(0xC0); // next line
	PINSEL1 = 0x01000000; /* P0.28 as AD0.1 */
  AD0CR = 0x00200402; /* ADC operational, 10-bits, 11 clocks for conversion */
	while(1)
	{
		AD0CR = AD0CR | (1<<24); /* Start Conversion */
		// LCD_STRING("2nd line");//second line  for testing
		while ( !(AD0DR1 & 0x80000000) ); /* Wait till DONE */
		LCD_STRING("3rd line");//third line  for testing 
		result = AD0DR1;
		result = (result>>6);
		result = (result & 0x000003FF);
		voltage = ( (result/1023.0) * 3.3 ); /* Convert ADC value to equivalent voltage */
		sprintf(volt, "Voltage=%.2f V  ", voltage);
		LCD_STRING(volt);
		memset(volt, 0, 18);
	}
}

